-- Frame management functions
Frames = {
  debug = false
}

local created = {}
local recycled = {
  Frame = {},
  Text = {},
  Texture = {},
  RiftButton = {},
  RiftCheckbox = {},
  RiftWindow = {},
  RiftTextfield = {}
}

function Frames:Create(frameType, namePrefix, parent)
  if type(recycled[frameType]) ~= 'table' then
    error('Invalid frame type: ' .. frameType)
  end

  local frames = recycled[frameType]
  local frame = table.remove(frames)
  if not frame then
    frame = UI.CreateFrame(frameType, namePrefix..tostring(#frames), parent)
    frame.frameType = frameType
    if Frames.debug then
      print("Created new "..frameType.." frame: "..frame:GetName())
    end
  else
    frame:SetParent(parent)
    if Frames.debug then
      print("Used recycled frame: "..frame:GetName())
    end
  end

  frame:SetVisible(true)

  table.insert(created, frame)
  return frame
end

function Frames:Recycle(frame)
  local children = {}
  local frameIndex

  for i, v in ipairs(created) do
    if v == frame then
      -- can't remove immediately since the next iteration would skip a frame
      frameIndex = i
    elseif v:GetParent() == frame then
      table.insert(children, v)
    end
  end

  -- TODO: What if frame wasn't found in created?

  if frameIndex ~= nil then
    table.remove(created, frameIndex)
  end

  table.insert(recycled[frame.frameType], frame)
  frame:SetVisible(false)
  frame:ClearAll()
  --frame:SetParent(nil)
  if Frames.debug then
    print("Recycled frame: "..frame:GetName())
  end

  for i, v in ipairs(children) do
    Frames:Recycle(v)
  end
end
