SimpleSK = {}

function SimpleSK:FindMissing(primaryList, raidMemberList)
    local missingList = {}
    local match = false
    for rIdx, rValue in ipairs(raidMemberList) do
        match = False
        for pIdx, pValue in ipairs(primaryList) do
            if pValue == rValue then
                match = true
                break
            end
        end

        -- Add any names that were not located in the primary list to the list of missing names
        if not match then
            table.insert(missingList, rValue)
        end

    end

    return missingList
end

function SimpleSK:AddNames(nameList, primaryList, method)
    local rndRange = #primaryList + 1
    local rndIdx

    for _, name in ipairs(nameList) do

        if method == "append" then
            table.insert(primaryList, name)

        elseif method == "random" then
            rndIdx = math.random(rndRange)
            table.insert(primaryList, rndIdx, name)

        else
            print("Unknown method to add name: " .. tostring(method))

        end

    end

    return primaryList, pIdx

end

function SimpleSK:ExtractOrderedListByName(primaryList, raidMemberList)
    if primaryList == nil or raidMemberList == nil then
        return nil, nil
    end

    -- Determine the location of the subList names in the primary list
    local indexList = {}

    ---- Create an list consisting of the raid members in their SK order
    local orderedList = {}
    for pIdx, pValue in ipairs(primaryList) do
        for rIdx, rValue in ipairs(raidMemberList) do
            if pValue == rValue then
                table.insert(indexList, pIdx)
                table.insert(orderedList, rValue)
            end
        end
    end

    return indexList, orderedList
end


function SimpleSK:ExtractOrderedListByIndex(primaryList, raidMemberIdxList)
    if primaryList == nil or raidMemberIdxList == nil then
        return nil, nil
    end

    ---- Create an list consisting of the raid members in their SK order
    local orderedList = {}
    for pIdx, pValue in ipairs(primaryList) do
        for rIdx, rValue in ipairs(raidMemberIdxList) do
            if pIdx == rValue then
                table.insert(orderedList, pValue)
            end
        end
    end

    return raidMemberIdxList, orderedList
end


function SimpleSK:SuicideIndex(oIndex, orderedList, suicideTrackingList)
    local name = orderedList[oIndex]

    -- Remove the name from its position in the current sublist
    table.remove(orderedList, oIndex)

    -- Append the name to its new position at the end of the current sublist
    table.insert(orderedList, name)

    -- Append the name to suicide tracking list
    table.insert(suicideTrackingList, name)

    -- This table will take the form {actionDesc, listTable, originalPos, newPos}
    local suicideEvent = {action = "suicide", listName = self.currentActiveListName, origin = oIndex, destination = #orderedList}

    return orderedList, suicideTrackingList, suicideEvent
end

function SimpleSK:UpdatePrimaryList(primaryList, indexList, orderedList)
    -- Always sort the list even if we expect a sorted list
    local sortedList = indexList
    table.sort(sortedList)

    for idx, ivalue in ipairs(sortedList) do
        primaryList[ivalue] =  orderedList[idx]
    end

    return primaryList
end

function SimpleSK:RepositionIndex(primaryList, fromIdx, toIdx)
    if toIdx < 1 or toIdx > #primaryList then
        print("Invalid destination index")
        return primaryList
    elseif fromIdx == toIdx then
        return primaryList
    end

    fromValue = primaryList[fromIdx]

    table.remove(primaryList, fromIdx)
    table.insert(primaryList, toIdx, fromValue)

    return primaryList
end

function SimpleSK:ShiftUp(primaryList, fromIdx)
    local toIdx = fromIdx - 1
    return SimpleSK:RepositionIndex(primaryList, fromIdx, toIdx)
end

function SimpleSK:ShiftDown(primaryList, fromIdx)
    local toIdx = fromIdx + 1
    return SimpleSK:RepositionIndex(primaryList, fromIdx, toIdx)
end

function SimpleSK:RemoveIndex(index, primaryList)
    table.remove(primaryList, index)
end

function SimpleSK:GetNameIndex(name, primaryList)
    local index = nil

    for nIdx, nValue in ipairs(primaryList) do
        if name == nValue then
            index = nIdx
            break
        end
    end

    return index
end
