function SimpleSK:CreateBaseWindow()
    local buttonWidth = 135

    -- Create our dialog window
    self.base = UI.CreateFrame("SimpleWindow", "SSK_BaseWindow", SimpleSK.context)

    -- Put the dialog in the middle of the screen
    self.base:SetPoint("CENTER", UIParent, "CENTER")

    -- Make the dialog window 3 buttons and some padding wide
    self.base:SetWidth(buttonWidth*3 + 40)
    self.base:SetHeight(551)

    -- Add a close button to the top right corner
    self.base:SetCloseButtonVisible(true)

    -- Hide the setup window
    SimpleSK.base:SetVisible(false)
end

function SimpleSK:CreateSetupFrame()
    -- Create our dialog window
    self.setupFrame = UI.CreateFrame("Frame", "SSK_SetupWindow", self.base)

    -- Anchor the frame to the base window
    self.setupFrame:SetAllPoints(self.base)
    self.setupFrame:SetLayer(self.base:GetLayer() + 5)

    -- Determine the layer
    local baseLayer = self.setupFrame:GetLayer()

    -- Add a button that imports the raid list
    self.setupFrameImportRaidButton = UI.CreateFrame("RiftButton", "ImportRaidButton", self.setupFrame)
    self.setupFrameImportRaidButton:SetText("Import Raid")
    self.setupFrameImportRaidButton:SetPoint("BOTTOMLEFT", self.setupFrame, "BOTTOMLEFT", 20, -20)
    self.setupFrameImportRaidButton:SetLayer(baseLayer + 10)
    self.setupFrameImportRaidButton.Event.LeftPress = function()
        SimpleSK:ImportRaid()
    end

    -- Add a button that switches to the active window
    self.setupFrameToggleActiveButton = UI.CreateFrame("RiftButton", "ToggleActiveButton", self.setupFrame)
    self.setupFrameToggleActiveButton:SetText("Active Lists")
    self.setupFrameToggleActiveButton:SetPoint("BOTTOMCENTER", self.setupFrame, "BOTTOMCENTER", 0, -20)
    self.setupFrameToggleActiveButton:SetLayer(baseLayer + 10)
    self.setupFrameToggleActiveButton.Event.LeftPress = function()
        SimpleSK:ShowActiveWindow()
        SimpleSK:ScrollToTop(SimpleSK.setupTableFrame, SimpleSK.setupMaskFrame)
    end

    -- Add a button that closes the window
    self.setupFrameOkButton = UI.CreateFrame("RiftButton", "OkButton", self.setupFrame)
    self.setupFrameOkButton:SetText("Close")
    self.setupFrameOkButton:SetPoint("BOTTOMRIGHT", self.setupFrame, "BOTTOMRIGHT", -20, -20)
    self.setupFrameOkButton:SetLayer(baseLayer + 10)
    self.setupFrameOkButton.Event.LeftPress = function()
        SimpleSK.setupFrame:SetVisible(false)
        SimpleSK.base:SetVisible(false)
        SimpleSK:ScrollToTop(SimpleSK.setupTableFrame, SimpleSK.setupMaskFrame)
    end

    -- Create a drop down frame
    self.setupSelectFrame = UI.CreateFrame("SimpleSelect", "SSK_SetupSelectFrame", self.setupFrame)
    self.setupSelectFrame:ResizeToDefault()
    self.setupSelectFrame:SetPoint("CENTER", self.setupFrame, "CENTER", 0, -200)
    self.setupSelectFrame:SetWidth(150)
    self.setupSelectFrame:SetBackgroundColor(0,0,0,1)
    self.setupSelectFrame:SetLayer(baseLayer + 1000)
    function self.setupSelectFrame.Event:ItemSelect(item)
        SimpleSK:SetupSelectNameClick(item)
    end

    -- Add a header row
    local headerFrame = UI.CreateFrame("Frame", "SSK_SetupHeaderFrame", self.setupFrame)
    headerFrame:SetPoint("TOPLEFT", self.base, "TOPLEFT", 50, 100)
    headerFrame:SetPoint("TOPRIGHT", self.base, "TOPRIGHT", -50,  100)

    ---- Create the header row
    local headerTable = Table(2, headerFrame)
    local row = headerTable:AddRow()

    -- Tint the header row
    --row:SetBackgroundColor(1, 1, 1, 0.25)

    -- Set the size and text of the header cells
    local cell = headerTable:GetCell(1,1)
    cell:SetText("Name")
    cell.valueFrame:SetFontSize(16)
    headerTable:RefreshLayout()

    -- Tighten the header frame around the table
    headerFrame:SetHeight(headerTable.totalHeight)

    -- Add checkbox for select all function
    cell = headerTable:GetCell(1,2)
    cell.valueFrame = UI.CreateFrame("RiftCheckbox", "SSK_HeaderCheckbox", cell.frame)
    self.selectAllCheckbox = cell.valueFrame
    cell.valueFrame:SetPoint("CENTER", cell.frame, "CENTER")
    cell.valueFrame.Event.CheckboxChange = function ()
        SimpleSK:ToggleSelectAll(cell.valueFrame)
    end

    -- Create a mask for the setup window
    self.setupMaskFrame = UI.CreateFrame("SimpleScrollView", "SSK_SetupMaskFrame", self.setupFrame)

    -- Anchor the mask frame to the base window
    self.setupMaskFrame:SetPoint("TOPLEFT", headerFrame, "BOTTOMLEFT")
    self.setupMaskFrame:SetPoint("BOTTOMRIGHT", self.base, "BOTTOMRIGHT", -50, -70)

    -- Create a frame inset from the main window to contain the setup table
    self.setupTableFrame = UI.CreateFrame("Frame", "SSK_SetupTableFrame", self.setupMaskFrame)
    self.setupMaskFrame:SetContent(self.setupTableFrame)
    self.setupMaskFrame:SetScrollInterval(20)

    local setupConfigTextureFrame = UI.CreateFrame("Texture", "SSK_SetupConfigTexture", self.setupFrame)
    setupConfigTextureFrame:SetPoint("CENTERRIGHT", self.setupFrame, "CENTERRIGHT", -50, -200)
    setupConfigTextureFrame:SetTexture("SimpleSK", "textures/cogs_icon&24.png")
    setupConfigTextureFrame:SetAlpha(0.75)

    function setupConfigTextureFrame.Event:MouseIn()
        self:SetAlpha(1)
    end

    function setupConfigTextureFrame.Event:MouseOut()
        self:SetAlpha(0.75)
    end

    setupConfigTextureFrame.Event.LeftClick = function()
        SimpleSK:ToggleConfigFrame()
    end

    -- Hide the setup window
    SimpleSK.setupFrame:SetVisible(false)

end

function SimpleSK:CreateActiveFrame()
    -- Create our dialog window
    self.activeFrame = UI.CreateFrame("Frame", "SSK_ActiveWindow", self.base)

    -- Put the dialog in the middle of the screen
    self.activeFrame:SetAllPoints(self.base)
    self.activeFrame:SetLayer(self.base:GetLayer() + 5)

    local baseLayer = self.activeFrame:GetLayer()

    -- Add a button that suicides the selected name
    self.activeFrameSuicideButton = UI.CreateFrame("RiftButton", "SuicideButton", self.activeFrame)
    self.activeFrameSuicideButton:SetText("Suicide")
    self.activeFrameSuicideButton:SetPoint("BOTTOMLEFT", self.activeFrame, "BOTTOMLEFT", 20, -20)
    self.activeFrameSuicideButton:SetLayer(baseLayer + 10)
    self.activeFrameSuicideButton.Event.LeftPress = function()
        SimpleSK:SuicideSelectedIndex(SimpleSK.activeTable)
        SimpleSK:ScrollToTop(SimpleSK.activeTableFrame, SimpleSK.activeMaskFrame)
    end

    -- Add a button that switches to the setup window
    self.activeFrameToggleSetupButton = UI.CreateFrame("RiftButton", "ToggleSetupButton", self.activeFrame)
    self.activeFrameToggleSetupButton:SetText("Setup Lists")
    self.activeFrameToggleSetupButton:SetPoint("BOTTOMCENTER", self.activeFrame, "BOTTOMCENTER", 0, -20)
    self.activeFrameToggleSetupButton:SetLayer(baseLayer + 10)
    self.activeFrameToggleSetupButton.Event.LeftPress = function()
        SimpleSK:ResetAllCheckboxes(SimpleSK.activeTable)
        SimpleSK:ScrollToTop(SimpleSK.activeTableFrame, SimpleSK.activeMaskFrame)
        SimpleSK:ShowSetupWindow()
    end

    -- Add a button that closes the window
    self.activeFrameOkButton = UI.CreateFrame("RiftButton", "OkButton", self.activeFrame)
    self.activeFrameOkButton:SetText("Close")
    self.activeFrameOkButton:SetPoint("BOTTOMRIGHT", self.activeFrame, "BOTTOMRIGHT", -20, -20)
    self.activeFrameOkButton:SetLayer(baseLayer + 10)
    self.activeFrameOkButton.Event.LeftPress = function()
        SimpleSK.activeFrame:SetVisible(false)
        SimpleSK.base:SetVisible(false)
        SimpleSK:ResetAllCheckboxes(SimpleSK.activeTable)
        SimpleSK:ScrollToTop(SimpleSK.activeTableFrame, SimpleSK.activeMaskFrame)
    end

    -- Create a drop down frame
    self.activeSelectFrame = UI.CreateFrame("SimpleSelect", "SSK_ActiveSelectFrame", self.activeFrame)
    self.activeSelectFrame:ResizeToDefault()
    self.activeSelectFrame:SetPoint("CENTER", self.setupFrame, "CENTER", 0, -200)
    self.activeSelectFrame:SetWidth(150)
    self.activeSelectFrame:SetBackgroundColor(0,0,0,1)
    self.activeSelectFrame:SetLayer(baseLayer + 1000)
    function self.activeSelectFrame.Event:ItemSelect(item)
        SimpleSK:ActiveSelectNameClick(item)
    end

    -- Add a header row
    local headerFrame = UI.CreateFrame("Frame", "SSK_ActiveHeaderFrame", self.activeFrame)
    headerFrame:SetPoint("TOPLEFT", self.base, "TOPLEFT", 50, 100)
    headerFrame:SetPoint("TOPRIGHT", self.base, "TOPRIGHT", -50,  100)

    ---- Create the header row
    local headerTable = Table(2, headerFrame)
    local row = headerTable:AddRow()

    -- Tint the header row
    --row:SetBackgroundColor(1, 1, 1, 0.25)

    -- Set the size and text of the header cells
    local cell = headerTable:GetCell(1,1)
    cell:SetText("Name")
    cell.valueFrame:SetFontSize(16)
    headerTable:RefreshLayout()

    -- Tighten the header frame around the table
    headerFrame:SetHeight(headerTable.totalHeight)

    -- Create a mask for the active window
    self.activeMaskFrame = UI.CreateFrame("SimpleScrollView", "SSK_ActiveMaskFrame", self.activeFrame)

    -- Anchor the mask frame to the base window
    self.activeMaskFrame:SetPoint("TOPLEFT", headerFrame, "BOTTOMLEFT")
    self.activeMaskFrame:SetPoint("BOTTOMRIGHT", self.base, "BOTTOMRIGHT", -50, -70)

    -- Create a frame inset from the main window to contain the active table
    self.activeTableFrame = UI.CreateFrame("Frame", "SSK_ActiveTableFrame", self.activeMaskFrame)
    self.activeMaskFrame:SetContent(self.activeTableFrame)
    self.activeMaskFrame:SetScrollInterval(20)

    --self.activeTableFrame:SetBackgroundColor(0,0,0,0.25)

    local activeConfigTextureFrame = UI.CreateFrame("Texture", "SSK_ActiveConfigTexture", self.activeFrame)
    activeConfigTextureFrame:SetPoint("CENTERRIGHT", self.activeFrame, "CENTERRIGHT", -50, -200)
    activeConfigTextureFrame:SetTexture("SimpleSK", "textures/cogs_icon&24.png")
    activeConfigTextureFrame:SetAlpha(0.75)

    function activeConfigTextureFrame.Event:MouseIn()
        self:SetAlpha(1)
    end

    function activeConfigTextureFrame.Event:MouseOut()
        self:SetAlpha(0.75)
    end

    activeConfigTextureFrame.Event.LeftClick = function()
        SimpleSK:ToggleConfigFrame()
    end

    -- Create undo button on active frame only
    local activeUndoTextureFrame = UI.CreateFrame("Texture", "SSK_ActiveUndoTexture", self.activeFrame)
    activeUndoTextureFrame:SetPoint("CENTERRIGHT", activeConfigTextureFrame, "CENTERLEFT", -10, 0)
    activeUndoTextureFrame:SetTexture("SimpleSK", "textures/undo_icon&24.png")
    activeUndoTextureFrame:SetAlpha(0.75)

    function activeUndoTextureFrame.Event:MouseIn()
        if #SimpleSK.undoList >= 1 then
            self:SetAlpha(1)
        end
    end

    function activeUndoTextureFrame.Event:MouseOut()
        self:SetAlpha(0.75)
    end

    activeUndoTextureFrame.Event.LeftClick = function()
        SimpleSK:UndoLastEvent()
    end


    -- Hide the active window
    self.activeFrame:SetVisible(false)

end