local function strjoin(delimiter, list)
    local len = #list
    if len == 0 then
        return ""
    end
    local string = list[1]
    for i = 2, len do
        string = string .. delimiter .. tostring(list[i])
    end
    return string
end

function SimpleSK:ConfigExportListClick()
    if not self.configSelectedListName or self.configSelectedListName == "" then return end
    self:ExportList(self.configSelectedListName, "text")

    -- Refresh the config frame to clear the selection
    self:RefreshConfigFrame()
end

function SimpleSK:ExportList(name, exportType)
    local list = self:GetListByName(name, self.pLists)
    local exportString

    -- Remember what was being export and what format
    self.exportListName = name
    self.exportType = exportType

    if exportType == "text" then
        exportString = self:ExportListAsText(list)
    elseif exportType == "html" then
        exportString = self:ExportListAsHtml(list)
    end

    self.baseTextAreaWindow:SetTitle("SimpleSK - Export")
    self.exportTextField:SetText(exportString)
    self.exportTextField:SetKeyFocus(true)
    if string.len(exportString) > 0 then
        self.exportTextField:SetSelection(0, string.len(self.exportTextField:GetText()))
    end
    self:GenerateExportSelectTable()

    self:ShowExportWindow()
end

function SimpleSK:ExportListAsText(list)
    return strjoin(string.char(13), list)
end

function SimpleSK:ExportListAsHtml(list)
    local exportString = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
    exportString = exportString .. "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">"
    exportString = exportString .. "<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"en\">"
    exportString = exportString .. "<head><title>" .. tostring(self.exportListName) .. "</title></head>"
    exportString = exportString .. "<body>" .. string.char(13)
    exportString = exportString .. "<ul>" .. string.char(13) .. "<li>"

    exportString = exportString .. strjoin("</li>" .. string.char(13) .. "<li>", list)

    exportString = exportString .. "</li>" .. string.char(13).. "</ul>" .. string.char(13)

    exportString = exportString .. "</body></html>"

    return exportString
end

function SimpleSK:ConfigExportSuicidesButtonClick()
    if not self.configSelectedListName or self.configSelectedListName == "" then return end
    local origName = self.configSelectedListName
    self:ExportSuicideEvents(origName)
    origName = nil

    -- Refresh the config frame to clear the selection
    self:RefreshConfigFrame()
end

function SimpleSK:ExportSuicideEvents(name)
    if not name or name == "" then
        print("Please provide the name of the list you export events for")
    end

    local list = self:GetListByName(name, self.sLists)

    if not list or #list == 0 then
        print("No suicide events for list " .. name)
        return
    end

    local string = strjoin(string.char(13), list)

    self.exportTextField:SetText(string)
    self.exportTextField:SetKeyFocus(true)
    self.exportTextField:SetSelection(0, string.len(self.exportTextField:GetText()))

    self:ShowExportWindow()
end

function SimpleSK:GenerateExportSelectTable()
    local typeNames = {"text", "html"}

    -- Fill in the drop down
    self.exportSelectFrame:SetItems(typeNames)

    -- Set the selected value if there is one but don't keep resetting it
    if self.exportType and not (self.exportType == self.exportSelectFrame:GetSelectedItem()) then
        self.exportSelectFrame:SetSelectedItem(self.exportType, true)
    end

end

function SimpleSK:ExportSelectType(item)
    self:ExportList(self.exportListName, item)
end
