local function CountPairs(pairTable)
    local count = 0
    for k, _ in pairs(pairTable) do
        count = count + 1
    end

    return count
end

local function strsplit(delimiter, text)
    local list = {}
    local pos = 1
    if string.find("", delimiter, 1) then -- this would result in endless loops
        error("delimiter matches empty string!")
    end
    while 1 do
        local first, last = string.find(text, delimiter, pos)
        if first then -- found?
            table.insert(list, string.sub(text, pos, first-1))
            pos = last+1
        else
            table.insert(list, string.sub(text, pos))
            break
        end
    end
    return list
end

local function strtrim(s)
    if not s then
        return nil
    end

    return s:match'^%s*(.*%S)' or ''
end

local COLORTABLE = {
    cleric = {0.60, 1.00, 0.25},
    mage = {0.65, 0.50, 1.00},
    rogue = {1.00, 0.80, 0.25},
    warrior = {1.00, 0.35, 0.45},
}

-- List of units we are interested in
local UNITLIST = {"player"}
for i = 1, 20 do
    table.insert(UNITLIST, string.format("group%02d", i))
end

-- Set up a config table
local CONFIG_TABLE = {
    insertRandom = {
        order = 10,
        type = "checkbox",
        label = "Insert names randomly",
        labelPos = "left",
        width = "full",
        get = function() return SimpleSK:GetAddMethod() end,
        set = function(value) SimpleSK:SetAddMethod() end,
    },
    raidOnly = {
        order = 20,
        type = "checkbox",
        label = "Only show raid members in active view",
        labelPos = "left",
        width = "full",
        get = function() return SimpleSK:GetActiveViewMethod() end,
        set = function(value) SimpleSK:SetActiveViewMethod(value) end,
    },
    suppressMessage = {
        order = 30,
        type = "checkbox",
        label = "Suppress welcome message",
        labelPos = "left",
        width = "full",
        get = function() return SimpleSK:GetSuppressMessageState() end,
        set = function(value) SimpleSK:SetSuppressMessageState(value) end,
    },
    listFontSize = {
        order = 40,
        type = "slider",
        label = "Font Size",
        labelPos = "left",
        width = "full",
        min = 10,
        max = 20,
        get = function() return SimpleSK:GetListFontSize() end,
        set = function(value) SimpleSK:SetListFontSize(value) end,
    }
}

function SimpleSK:RefreshSetupWindow()
    -- Give a title to the window
    self.base:SetTitle("SimpleSK - Setup List")

    -- Update raid member names
    self:GetRaidNames()

    -- Update pList
    self.pList = self:GetListByName(self.currentSetupListName, self.pLists)

    -- Get the list of currently selected names
    self.currentList = self:GetListByName(self.currentSetupListName, self.currentLists)

    -- Put check marks next to the names of the currently selected names
    self:RefreshCheckboxesByIndex(self.currentList, self.setupTable)

    -- Update the setup table content
    self:GenerateSetupTable(self.pList)

    -- Tighten the setup table frame around the table
    self.setupTableFrame:SetHeight(self.setupTable.totalHeight)

end

function SimpleSK:ShowSetupWindow()
    -- Refresh the content
    self:RefreshSetupWindow()

    -- Remember which window was showing last
    self.windowState = "setup"

    -- Clear visibility of other frames
    self:HideAllFrames()

    -- Force visibility of setup frame
    self.setupFrame:SetVisible(true)
    self.base:SetVisible(true)
end

function SimpleSK:CreateSetupTable()
    self.setupTable = Table(2, self.setupTableFrame)
end

function SimpleSK:RefreshActiveWindow()
    -- Give a title to the window
    self.base:SetTitle("SimpleSK - Active List")

    -- Update raid member names
    self:GetRaidNames()

    -- Update the ordered lists
    local iList, oList
    for name, pList in pairs(self.pLists) do
        iList, oList = self:ExtractOrderedListByIndex(self.pLists[name], self.currentLists[name])
        self.iLists = self:UpdateLists(self.iLists, name, iList)
        self.oLists = self:UpdateLists(self.oLists, name, oList)
    end

    -- Grab the current oList or default to setup name if it was active
    if not self.currentActiveListName and self.currentSetupListName ~= "" then
        self.currentActiveListName = self.currentSetupListName
        self.oList = self:GetListByName(self.currentSetupListName, self.oLists)
        self.sList = self:GetListByName(self.currentSetupListName, self.sLists)
    else
        self.oList = self:GetListByName(self.currentActiveListName, self.oLists)
        self.sList = self:GetListByName(self.currentActiveListName, self.sLists)
    end

    -- Update the active table content
    self:GenerateActiveTable(self.oList)

    -- Tighten the active table frame around the table
    self.activeTableFrame:SetHeight(self.activeTable.totalHeight)

end

function SimpleSK:ShowActiveWindow()
    -- Refresh the content
    self:RefreshActiveWindow()

    -- Remember which window was showing last
    self.windowState = "active"

    -- Clear visibility of other frames
    self:HideAllFrames()

    -- Force visibility of active frame
    self.activeFrame:SetVisible(true)
    self.base:SetVisible(true)
end

function SimpleSK:CreateActiveTable()
    self.activeTable = Table(2, self.activeTableFrame)
end

function SimpleSK:GenerateSetupTable(nameList)
    -- Works with empty tables, but should return if the table is nil
    if not nameList then return end

    local mainColor = {0, 0, 0, 0}
    local altColor = {0, 0, 0, 0.25}
    local genericColor = {1, 1, 1}

    local pTable = self.setupTable
    local row, cell, idx
    local minRows

    local extraRows = 1

    if #nameList > 0 then
        self.selectAllCheckbox:SetEnabled(true)
    else
        self.selectAllCheckbox:SetEnabled(false)
    end


    -- Determine minimum number of rows required
    if nameList then
        minRows = #nameList + extraRows
    else
        minRows = 5 + extraRows
    end

    -- Check that the setup table has enough rows and add more if necessary
    while pTable:GetRowCount() < minRows do   -- for the header
        row = pTable:AddRow()

        idx = pTable:GetRowCount()

        -- Display checkbox in 2nd column
        cell = row:GetCell(2)
        cell.valueFrame = UI.CreateFrame("RiftCheckbox", "SSK_SetupCheckBox_" .. idx, cell.frame)
        cell.valueFrame:SetPoint("CENTER", cell.frame, "CENTER")
        function cell.valueFrame.Event:CheckboxChange()
            local listName = SimpleSK.currentSetupListName
            SimpleSK:UpdateCurrentList(listName)
        end

    end

    local name, textWidth, classColor, unitID
    for idx = 1, pTable:GetRowCount() do
        -- Get the row object
        row = pTable:GetRow(idx)
        cell = pTable:GetCell(idx, 1)

        if nameList and nameList[idx] then
            -- Attach the name and index to the frame
            name = nameList[idx]
            cell.frame.name = name
            cell.frame.index = idx

            -- Set name
            cell:SetText(string.format("%d. %s", idx, name))
            cell:SetAlignment("LEFT", "CENTER")

            -- Set the font size
            cell.valueFrame:SetFontSize(self.listFontSize)

            -- Set color
            if self.playerListByName and self.playerListByName[name] then
                unitID = self.playerListByName[name]
                classColor = COLORTABLE[self.playerListByID[unitID].calling]
                cell.valueFrame:SetFontColor(unpack(classColor or genericColor))
            else
                cell.valueFrame:SetFontColor(unpack(genericColor))
            end

            -- Tint alternate rows darker
            if (idx) % 2 ~= 0 then
                row:SetBackgroundColor(unpack(altColor))
            end

            -- Make entire row clickable
            function cell.frame.Event:LeftClick()
                local index = self.index
                SimpleSK:EditIndexClick(index)
            end

            -- Enable the checkbox in case it was disabled
            cell = pTable:GetCell(idx, 2)
            cell.valueFrame:SetEnabled(true)

            -- Show row
            row.frame:SetVisible(true)

        elseif (not nameList and idx <= 1) or (nameList and idx == #nameList + extraRows) then
            -- Clear the name attached to the frame
            cell.frame.name = nil

            -- Set the text
            if self.currentSetupListName ~= "" then
                cell:SetText("Click to add name")
            else
                cell:SetText("Select list from dropdown")
            end
            cell:SetAlignment("LEFT", "CENTER")
            cell.valueFrame:SetFontSize(self.listFontSize)

            -- Set color
            cell.valueFrame:SetFontColor(unpack(genericColor))

            -- Tint alternate rows darker
            if (idx) % 2 ~= 0 then
                row:SetBackgroundColor(unpack(altColor))
            end

            -- Set the function attached to the left click event
            if self.currentSetupListName ~= "" then
                cell.frame.Event.LeftClick = function()
                    SimpleSK:AddNameClick()
                end

            else
                cell.frame.Event.LeftClick = nil

            end

            -- Disable the check box
            cell = pTable:GetCell(idx, 2)
            cell.valueFrame:SetChecked(false)
            cell.valueFrame:SetEnabled(false)

            -- Show row
            row.frame:SetVisible(true)

        else
            -- Clear the name attached to the frame
            cell.frame.name = nil

            -- Clear the text
            cell:SetText("")

            -- Clear the color
            cell.valueFrame:SetFontColor(unpack(genericColor))

            -- Reset background color to default
            row:SetBackgroundColor(unpack(mainColor))

            -- Clear any events
            cell.frame.Event.LeftClick = nil

            -- Hide the row
            row.frame:SetVisible(false)
        end

    end

    -- Refresh the layout to do all the resizing etc
    pTable:RefreshLayout()

end

function SimpleSK:GenerateSelectTable()
    local listNames = {}

    -- Create a list of all the primary lists
    for k,v in pairs(self.pLists) do
        table.insert(listNames, k)
    end

    -- Sort the list
    table.sort(listNames)

    -- Fill in the drop down
    self.setupSelectFrame:SetItems(listNames)
    self.activeSelectFrame:SetItems(listNames)

    -- Set the selected value if there is one
    if self.currentSetupListName and string.len(self.currentSetupListName) > 0 and not (self.currentSetupListName == self.setupSelectFrame:GetSelectedItem()) then
        self.setupSelectFrame:SetSelectedItem(self.currentSetupListName)
    end

    if self.currentSetupListName and string.len(self.currentSetupListName) > 0 and not (self.currentSetupListName == self.activeSelectFrame:GetSelectedItem()) then
        self.activeSelectFrame:SetSelectedItem(self.currentSetupListName)
    end

end

function SimpleSK:SetupSelectNameClick(name)
    -- Update the current setup list name
    self.currentSetupListName = name

    -- Change active list name in sync with setup list
    self.currentActiveListName = name

    -- Change to the selected list
    self.pList = self:GetListByName(self.currentSetupListName, self.pLists)

    self:RefreshSetupWindow()
end

function SimpleSK:AddList(name)
    name = strtrim(name)
    self.currentSetupListName = name
    self.pList = {}
    self.pLists = self:UpdateLists(self.pLists, name, self.pList)

    self:GenerateSelectTable()
end

function SimpleSK:GenerateActiveTable(nameList)
    -- Works with empty tables, but should return if the table is nil
    if not nameList then return end

    local mainColor = {0, 0, 0, 0}
    local altColor = {0, 0, 0, 0.25}
    local genericColor = {0.75, 0.75, 0.75}

    local pTable = self.activeTable
    local row, cell, idx
    local minRows

    local extraRows = 0

    -- Determine minimum number of rows required
    if #nameList > 0 then
        minRows = #nameList + extraRows
    elseif #nameList == 0 then
        minRows = 5 + extraRows
    else
        minRows = 5 + extraRows
    end

    -- Check that the setup table has enough rows and add more if necessary
    while pTable:GetRowCount() < minRows do   -- for the header
        row = pTable:AddRow()

        idx = pTable:GetRowCount()

        -- Display checkbox in 2nd column
        cell = row:GetCell(2)
        cell.valueFrame = UI.CreateFrame("RiftCheckbox", "SSK_ActiveCheckBox_" .. idx, cell.frame)
        cell.valueFrame:SetPoint("CENTER", cell.frame, "CENTER")
        cell.valueFrame.Event.CheckboxChange = function()
            SimpleSK:ToggleEnableUncheckedBoxes(pTable)
        end

    end

    local name, textWidth, classColor, unitID, nIdx, match
    nIdx = 0
    for idx = 1, pTable:GetRowCount() do
        -- Get the row object
        row = pTable:GetRow(idx)
        cell = pTable:GetCell(idx, 1)

        -- Reset name
        name = nil
        nIdx = nIdx + 1

        if self.activeViewMethod then
            match = false
            while #nameList > 0  and nIdx <= #nameList and nameList[nIdx] and not match do
                name = nameList[nIdx]

                -- Check if the name is in the list of raiders
                for raidName, unitId in pairs(self.playerListByName) do
                    if raidName == name then
                        match = true
                        break
                    end
                end

                -- If a match was found exit this while loop
                if match then break end

                -- Otherwise increment the counter
                nIdx = nIdx + 1
            end
        else
            -- Don't care about finding raid members only, default to true
            match = true
        end

        if #nameList > 0 and nameList[nIdx] and match then
            -- Get name
            name = nameList[nIdx]

            -- Attach name to frame so that it can be retrieved inside the function
            cell.frame.name = name
            cell.frame.oIndex = nIdx

            -- Tint alternate rows darker
            if (idx) % 2 ~= 0 then
                row:SetBackgroundColor(unpack(altColor))
            end

            -- Display name in 1st column
            cell:SetText(string.format("%d. %s", nIdx, name))
            cell:SetAlignment("LEFT", "CENTER")

            -- Set the font size
            cell.valueFrame:SetFontSize(self.listFontSize)

            -- Set color
            if self.playerListByName and self.playerListByName[name] then
                unitID = self.playerListByName[name]
                classColor = COLORTABLE[self.playerListByID[unitID].calling]
                cell.valueFrame:SetFontColor(unpack(classColor or genericColor))
            else
                cell.valueFrame:SetFontColor(unpack(genericColor))
            end

            -- Re-enable the check box
            cell = pTable:GetCell(idx, 2)
            cell.valueFrame:SetEnabled(true)

            -- Ensure that it is visible in case we hid it previously
            row.frame:SetVisible(true)


        elseif #nameList == 0 and idx <= 5 + extraRows then
            -- Tint alternate rows darker
            if (idx) % 2 ~= 0 then
                row:SetBackgroundColor(unpack(altColor))
            end

            -- Detach name from frame
            cell.frame.name = nil
            cell.frame.oIndex = nil

            -- Display dash in 1st column
            cell = pTable:GetCell(idx, 1)
            cell:SetText("-")
            cell:SetAlignment("CENTER", "CENTER")

            -- Set the font size
            cell.valueFrame:SetFontSize(self.listFontSize)

            -- Set color
            cell.valueFrame:SetFontColor(unpack(genericColor))

            -- Disable the check box
            cell = pTable:GetCell(idx, 2)
            cell.valueFrame:SetEnabled(false)

            -- Ensure that it is visible in case we hid it previously
            row.frame:SetVisible(true)

        else
            -- Reset background color to default
            row:SetBackgroundColor(unpack(mainColor))

            -- Detach name from frame
            cell.frame.name = nil
            cell.frame.oIndex = nil

            -- Set text to empty string
            cell = pTable:GetCell(idx, 1)
            cell:SetText("")
            cell:SetAlignment("LEFT", "CENTER")

            -- Set color
            cell.valueFrame:SetFontColor(unpack(genericColor))

            -- Disnable the check box
            cell = pTable:GetCell(idx, 2)
            cell.valueFrame:SetEnabled(false)

            -- Just hide the row
            row.frame:SetVisible(false)
        end

    end

    -- Refresh the layout to do all the resizing etc
    pTable:RefreshLayout()

end

function SimpleSK:ActiveSelectNameClick(name)
    -- Update the current active list name
    self.currentActiveListName = name

    -- Change the setup list in sync
    self.currentSetupListName = name

    -- Update the ordered list and the suicide list
    self.oList = self:GetListByName(self.currentActiveListName, self.oLists)
    self.sList = self:GetListByName(self.currentActiveListName, self.sLists)

    self:RefreshActiveWindow()
end

function SimpleSK:UpdateCurrentList(listName)
    -- Get the list of check box cell frames
    local pTable = self.setupTable
    local nameCells = pTable:GetColumn(1)
    local checkboxCells = pTable:GetColumn(2)
    local cellState

    -- Append each checked name to a list of checked names
    local checkedIdx = {}
    for i, cell in ipairs(checkboxCells) do
        if cell.valueFrame then
            cellState = cell.valueFrame:GetChecked()
            if cellState then
                table.insert(checkedIdx, nameCells[i].frame.index)
            end
        end
    end

    -- Store the list of names in the current list
    self.currentLists = self:UpdateLists(self.currentLists, listName, checkedIdx)

end

function SimpleSK:RefreshCheckboxesByIndex(idxList, pTable)
    if idxList == nil then
        idxList = {}
    end

    -- First disable all checkboxes
    local checkboxList = pTable:GetColumn(2)
    for cIdx, cValue in ipairs(checkboxList) do
        if cValue.valueFrame and cValue.valueFrame.SetChecked then
            cValue.valueFrame:SetChecked(false)
        end
    end

    -- Get the list of indices currently displayed
    local nameList = pTable:GetColumn(1)

    for rIdx, rValue in ipairs(idxList) do
        for nIdx, nValue in ipairs(nameList) do
            -- Find the index associated with the row
            local index
            if nValue.frame.index then
                index = nValue.frame.index
            end

            -- If the name matches set the checkbox to true
            if rValue and index and rValue == index then
                checkboxList[nIdx].valueFrame:SetChecked(true)
                break
            end

        end
    end

end

function SimpleSK:ReturnCheckedIndex(pTable)
    local checkboxList = pTable:GetColumn(2)

    -- Go through list to see if any are enabled
    local enabledIdx
    for cIdx, cValue in ipairs(checkboxList) do
        if cValue.valueFrame and cValue.valueFrame.GetChecked then
            if cValue.valueFrame:GetChecked() then
                enabledIdx = cIdx
                break
            end
        end
    end

    return enabledIdx
end

function SimpleSK:GetSuicideOrderedIndex(pTable)
    local checkboxList = pTable:GetColumn(2)

    -- Go through list to see if any are enabled
    local oIndex, nameCell
    for cIdx, cValue in ipairs(checkboxList) do
        if cValue.valueFrame and cValue.valueFrame.GetChecked then
            if cValue.valueFrame:GetChecked() then
                nameCell = pTable:GetCell(cIdx, 1)
                oIndex = nameCell.frame.oIndex
                break
            end
        end
    end

    return oIndex
end

function SimpleSK:ToggleEnableUncheckedBoxes(pTable)
    -- Poor mans radio button
    -- Disable all other checkboxes if one is selected
    local checkboxList = pTable:GetColumn(2)

    local enabledIdx = self:ReturnCheckedIndex(pTable)

    for cIdx, cValue in ipairs(checkboxList) do
        if cValue.valueFrame and cValue.valueFrame.GetChecked then
            if enabledIdx then
                if cIdx ~= enabledIdx then
                    cValue.valueFrame:SetEnabled(false)
                end

            else
                cValue.valueFrame:SetEnabled(true)

            end
        end
    end

end

function SimpleSK:ResetAllCheckboxes(pTable)
    local checkboxList = pTable:GetColumn(2)

    -- Go through list to see if any are enabled
    local enabledIdx
    for cIdx, cValue in ipairs(checkboxList) do
        if cValue.valueFrame and cValue.valueFrame.GetChecked then
            cValue.valueFrame:SetEnabled(true)
            cValue.valueFrame:SetChecked(false)
        end
    end

    return enabledIdx

end

function SimpleSK:SuicideSelectedIndex(pTable)
    -- Determine index of person selected
    local oIndex = self:GetSuicideOrderedIndex(pTable)

    if oIndex then
        -- Call the suicide function
        local oList, sList, eventTable
        oList, sList, eventTable = self:SuicideIndex(oIndex, self.oList, self.sList)
        self.oLists = self:UpdateLists(self.oLists, self.currentActiveListName, oList)
        self.sLists = self:UpdateLists(self.sLists, self.currentActiveListName, sList)

        -- Store the event table in the undo
        table.insert(self.undoList, eventTable)

        -- Update the primary list
        local activePrimaryList = self:GetListByName(self.currentActiveListName, self.pLists)
        local activeIndexList = self:GetListByName(self.currentActiveListName, self.iLists)
        activePrimaryList = self:UpdatePrimaryList(activePrimaryList, activeIndexList, oList)
        self.pLists = self:UpdateLists(self.pLists, self.currentActiveListName, activePrimaryList)

        -- Clear all the checkboxes
        self:ResetAllCheckboxes(pTable)
    end

    -- Refresh the active window
    self:RefreshActiveWindow()

end

function SimpleSK:AddNameClick()
    -- Set focus and selection
    local text = "Player Name"
    self.inputTextField:SetKeyFocus(true)
    self.inputTextField:SetText(text)
    self.inputTextField:SetSelection(0, string.len(self.inputTextField:GetText()))

    local returnFn = function()
        -- Hide the window
        SimpleSK.inputWindow:SetVisible(false)
        SimpleSK.baseNameWindow:SetVisible(false)

        -- Grab the input value
        SimpleSK:AddName(SimpleSK.inputTextField:GetText())

        -- Clear the field and focus
        SimpleSK.inputTextField:SetText("")
        SimpleSK.inputTextField:SetKeyFocus(false)

        -- Update the setup window list
        SimpleSK:RefreshSetupWindow()
    end

    -- Pop up the textfield window
    self:InputFrame(returnFn, returnFn)
end

function SimpleSK:SelectRaidMembers()
    -- Get raid list
    local rList = self:GetRaidNames()

    -- Get the current list of names
    local currentNames = self:GetListByName(self.currentSetupListName, self.pLists)

    -- Find out who is in the raid that is also in our list (name compare unfortunately)
    local idxlist, orderedNamelist = self:ExtractOrderedListByName(currentNames, rList)

    -- Get the currently selected indices
    local currentIdxList = self:GetListByName(self.currentSetupListName, self.currentLists)

    -- Append currently selected indices to the indices of the raid names
    -- Ditto for the actual names
    for _, index in ipairs(currentIdxList) do
        table.insert(idxlist, index)
        table.insert(rList, currentNames[index])
    end

    -- May need to get rid of duped indices here

    -- Put check marks next to the names
    self:RefreshCheckboxesByIndex(idxlist, self.setupTable)

    -- Optional return value if needed for missing
    return rList
end

function SimpleSK:ImportRaid()
    -- Get a list of raid members and currently selected names
    local rList = self:SelectRaidMembers()

    -- Determine which names are missing
    local missing = self:FindMissing(self.pList, rList)
    self.missing = missing

    -- Pop up dialog asking if they want to add missing names
    local messageString = "Would you like to add members not on the list?"

    local yesFn = function()
        SimpleSK.messageFrame:SetVisible(false)
        SimpleSK.baseNameWindow:SetVisible(false)
        _, SimpleSK.iList = SimpleSK:AddNames(SimpleSK.missing, SimpleSK.pList, SimpleSK.addMethod)
        SimpleSK:UpdateLists(SimpleSK.iLists, SimpleSK.currentSetupNameList, SimpleSK.iList)
        SimpleSK.missing = {}
        -- Must refresh window before selecting raid members to refresh pTable
        SimpleSK:RefreshSetupWindow()
        SimpleSK:SelectRaidMembers()
    end

    local noFn = function()
        SimpleSK.messageFrame:SetVisible(false)
        SimpleSK.baseNameWindow:SetVisible(false)
        SimpleSK.missing = {}
    end

    if #missing > 0 then
        self:MessageDialog(messageString, "Yes", yesFn, "No", noFn)
    else
        self:RefreshSetupWindow()
    end

end

function SimpleSK:RefreshWindow()
    if self.windowState == "setup" then
        self:RefreshSetupWindow()

    elseif self.windowState == "active" then
        self:RefreshActiveWindow()

    else
        self:RefreshSetupWindow()
    end

end

function SimpleSK:ShowWindow()
    if self.windowState == "setup" then
        self:ShowSetupWindow()

    elseif self.windowState == "active" then
        self:ShowActiveWindow()

    else
        self:ShowSetupWindow()
    end

end

-- Define default values. Default value must never be nil
local profile = {
    pLists = {},
    currentSetupListName = "",
    addMethod = "append",
    activeViewMethod = false,
    suppressMessage = false,
    listFontSize = 12,
}

local function Initialize(addonName)
    -- Do not put anything that can crash after this point
    if addonName ~= "SimpleSK" then
        return
    end

    -- Set default values
    if not SimpleSK_SavedVariables then
        -- Has to be global for toc
        SimpleSK_SavedVariables = {}
    end

    for key, value in pairs(profile) do
        if not SimpleSK_SavedVariables[key] then
            SimpleSK[key] = value
        else
            SimpleSK[key] = SimpleSK_SavedVariables[key]
        end
    end

    -- Do not put anything before this point

    -- Seed the RNG with the time
    local time = Inspect.Time.Real()
    math.randomseed(time)

    SimpleSK.oLists = {}
    SimpleSK.iLists = {}
    SimpleSK.sLists = {}
    SimpleSK.missing = {}
    SimpleSK.undoList = {}

    SimpleSK.currentActiveListName = nil
    SimpleSK.pList = SimpleSK:GetListByName(SimpleSK.currentSetupListName, SimpleSK.pLists)
    SimpleSK.scrollYOffset = 0

    --SimpleSK.rList = {}
    SimpleSK.currentLists = {}
    SimpleSK.currentList = {}
    --SimpleSK.rList = {"Winky", "Thang", "Sushi", "Seafood"}
    SimpleSK.windowState = "setup"

    -- Initialize frames
    SimpleSK:InitializeFrames()

    -- Print annoying spammy welcome message until there is a nice common interface
    if not SimpleSK.suppressMessage then
        print("SimpleSK loaded. Type /ssk or /ssk help for options.")
    end

    -- Initialize the select lists
    SimpleSK:GenerateSelectTable()

end

function SimpleSK:InitializeFrames()
    -- Frame Creation
    -- Create our context
    self.context = UI.CreateContext("SSK_Context")

    -- Create base window
    self:CreateBaseWindow()

    -- Create window for setup view
    self:CreateSetupFrame()

    -- Create a setup table object
    self:CreateSetupTable()

    -- Create window for setup view
    self:CreateActiveFrame()

    -- Create a active table object
    self:CreateActiveTable()

    -- Create a base Import/Export window
    self:CreateBaseImportWindow()

    -- Create a frame to export to clipboard
    self:CreateExportFrame()

    -- Create a frame to import from clipboard
    self:CreateImportFrame()

    -- Create configuration window
    self:CreateConfigWindow()

    -- Create configuration frame
    self:CreateConfigFrame()

    -- Create base textfield window
    self:CreateBaseNameWindow()

    -- Create message frame
    self:CreateMessageFrame()

    -- Create generic input frame
    self:CreateInputFrame()
end

function SimpleSK:SaveVars()
    for key, _ in pairs(profile) do
        -- Avoid overwriting with nil in the event that the addon crashes
        if not (SimpleSK[key] == nil) then
            SimpleSK_SavedVariables[key] = SimpleSK[key]
        end
    end
end

function SimpleSK:AddName(name)
    name = strtrim(name)
    if name == "" or not self.pList then
        return nil
    end

    local nameList = {name}

    -- Add the name to the primary list
    _, self.iList = self:AddNames(nameList, self.pList, self.addMethod)
    self:UpdateLists(self.iLists, self.currentSetupListName, self.iList)
    SimpleSK_SavedVariables.pLists = self.pLists

    -- Refresh the setup window
    self:RefreshSetupWindow()
end

function SimpleSK:EditIndex(newName)
    -- Retrieve the index of the name from the namelist being edited
    local idx = self.editIndex
    local nameList = self.editNameList

    newName = strtrim(newName)

    -- Delete the entry if the name is blank
    if newName == "" or newName == nil then
        self:RemoveIndex(idx, nameList)

    else
        nameList[idx] = newName
    end

    -- Update the saved variables with the new list
    SimpleSK_SavedVariables.pLists = self.pLists

    -- Refresh the setup table
    self:RefreshSetupWindow()
end

function SimpleSK:EditIndexClick(index)
    -- Remember the name and the list it came friom
    self.editIndex = index
    self.editNameListName = self.currentSetupListName

    -- Retrieve the list of names
    self.editNameList = self:GetListByName(self.editNameListName, self.pLists)

    -- Retrieve the name at the index
    local name = tostring(self.editNameList[index])

    -- Set focus and selection
    self.inputTextField:SetKeyFocus(true)
    self.inputTextField:SetText(name)
    self.inputTextField:SetSelection(0, string.len(self.inputTextField:GetText()))

    local returnFn = function()
        -- Hide the window
        SimpleSK.inputWindow:SetVisible(false)
        SimpleSK.baseNameWindow:SetVisible(false)

        -- Grab the input value
        SimpleSK:EditIndex(SimpleSK.inputTextField:GetText())

        -- Clear the field and focus
        SimpleSK.inputTextField:SetText("")
        SimpleSK.inputTextField:SetKeyFocus(false)

        -- Update the setup window list
        SimpleSK:RefreshSetupWindow()
    end

    -- Pop up the textfield window
    self:InputFrame(returnFn, returnFn)
end

function SimpleSK:GetRaidNames()
    -- Obtain a list of the unitIDs of the units we are interested in
    local unitIdList = {}
    local unitID
    for idx, unitSpecifier in ipairs(UNITLIST) do
        unitID = Inspect.Unit.Lookup(unitSpecifier)
        if unitID then
            unitIdList[unitID] = unitSpecifier
        end
    end

    -- Obtain the details of all the units we are interested in
    local unitDetails = Inspect.Unit.Detail(unitIdList)

    -- Check that they are players and if so add them to the tracker
    local playerList = {}
    local playerListByName = {}
    local playerListByID = {}
    for key, value in pairs(unitDetails) do
        if value["player"] then
            table.insert(playerList, value["name"])
            playerListByName[value["name"]] = key
        end
    end

    -- Store for later use
    self.playerListByName = playerListByName
    self.playerListByID = unitDetails

    return playerList
end

function SimpleSK:GetListByName(name, lists)
    if not name or not lists or not lists[name] or name == "" then
        return {}
    end

    return lists[name]
end

function SimpleSK:UpdateLists(lists, name, list)
    if name and name ~= "" and lists then
        lists[name] = list
    end

    return lists
end

function SimpleSK:DeleteList(name)
    if not name then return end

    -- Zap the entry
    self.pLists[name] = nil

    -- Update the current setup list
    if self.currentSetupListName == name then
        self.currentSetupListName = ""
        self.pList = self:GetListByName(self.currentSetupListName, self.pLists)
        self:RefreshSetupWindow()
    end

    -- Update the entries on the select table
    self:GenerateSelectTable()

end

local function SlashHandler(arg)
    local cmds = strsplit(" ", arg)
    local cmd = cmds[1]
    local name
    if cmd == "listdel" then
        name = cmds[2]
        SimpleSK:DeleteList(name)

    elseif cmd == "export" then
        name = cmds[2]
        SimpleSK:ExportList(name, "text")

    elseif cmd == "exporthtml" then
        name = cmds[2]
        SimpleSK:ExportList(name, "html")

    elseif cmd == "exportsuicides" then
        name = cmds[2]
        SimpleSK:ExportSuicideEvents(name)

    elseif cmd == "import" then
        name = cmds[2]
        SimpleSK:ImportList(name)

    elseif cmd == "config" then
        SimpleSK:ShowConfigFrame()

    elseif cmd == "help" then
        SimpleSK:PrintHelp()

    elseif cmd:len() ~= 0 then
        SimpleSK:PrintHelp()

    else
        if next(SimpleSK.pLists) == nil then
            SimpleSK:ShowConfigFrame()
        else
            SimpleSK:ShowWindow()
        end

    end

end

function SimpleSK:PrintHelp()
    print("SimpleSK is a Suicide Kings list maintainer")
    print("/ssk config - open the configuration window")
    print("/ssk listdel <listname> - delete the list")
    print("/ssk export <listname> - export the list")
    print("/ssk exporthtml <listname> - export the list")
    print("/ssk exportsuicides <listname> - export the list of suicide events in the order that they occured")
    print("/ssk import <listname> - import the list")
    print("/ssk help - prints this message")
    print("/ssk - opens the list window")
end

function SimpleSK:ImportList(name)
    if not name or name == "" then
        print("Please provide a name for the list")
        return
    end

    -- Store the name of the list we are importing to
    self.importListName = name

    -- Pop up a blank import window
    self.baseTextAreaWindow:SetTitle("SimpleSK - Import")
    self.importTextField:SetText("")
    self.importTextField:SetKeyFocus(true)
    self:ShowImportWindow()
end

function SimpleSK:ShowExportWindow()
    self.exportFrame:SetVisible(true)
    self.baseTextAreaWindow:SetVisible(true)
end

function SimpleSK:ShowImportWindow()
    self.importFrame:SetVisible(true)
    self.baseTextAreaWindow:SetVisible(true)
end

function SimpleSK:UpdateListsFromImport(string)
    -- Do not accept blank strings
    if not string or string == "" then
        return
    end

    -- Assume carriage return separated list
    local listTable = strsplit(string.char(13), string)

    -- Remove any ""
    for i, v in ipairs(listTable) do
        if v == "" then
            table.remove(listTable, i)
        end
    end

    -- Trim whitespace
    for i, v in ipairs(listTable) do
        listTable[i] = strtrim(listTable[i])
    end

    self.pLists = self:UpdateLists(self.pLists, self.importListName, listTable)
    self:GenerateSelectTable()
end

function SimpleSK:HideAllFrames()
    self.base:SetVisible(false)
    self.baseConfigWindow:SetVisible(false)
    self.baseNameWindow:SetVisible(false)
    self.baseTextAreaWindow:SetVisible(false)
    self.setupFrame:SetVisible(false)
    self.activeFrame:SetVisible(false)
    self.messageFrame:SetVisible(false)
    self.exportFrame:SetVisible(false)
    self.importFrame:SetVisible(false)
    self.configFrame:SetVisible(false)
end

function SimpleSK:MessageDialog(message, yesText, yesFn, noText, noFn)
    local maxWidth = 350
    self.messageText:SetText(message)
    self.messageText:SetFontSize(16)
    local width = self.messageText:GetWidth()
    if width > maxWidth then
        self.messageText:SetWidth(maxWidth)
        self.messageText:SetWordwrap(true)
    else
        self.messageText:SetWidth(width)
    end
    local height = self.messageText:GetHeight()

    self.messageText:SetHeight(height)

    self.messageYesButton:SetText(yesText)
    self.messageYesButton.Event.LeftPress = yesFn

    self.messageNoButton:SetText(noText)
    self.messageNoButton.Event.LeftPress = noFn

    self.messageFrame:SetVisible(true)
    self.baseNameWindow:SetVisible(true)
end

function SimpleSK:RefreshConfigFrame()
    -- Update the title
    self.base:SetTitle("SimpleSK - Configuration")

    -- Generate the content
    self:GenerateConfigTable()
    self:GenerateConfigListNamesTable()

    -- Clear the selection
    self.configListNamesFrame:SetSelectedItem(nil)
end

function SimpleSK:GenerateConfigTable()
    Library.LibSimpleWidgets.Layout(CONFIG_TABLE, SimpleSK.configTableFrame)
end

function SimpleSK:GenerateConfigListNamesTable()
    -- Create a list of all the primary list names
    local listNames = {}

    for k,v in pairs(self.pLists) do
        table.insert(listNames, k)
    end

    -- Sort the list of names
    table.sort(listNames)

    -- Fill in the list frame
    self.configListNamesFrame:SetItems(listNames)
end

-- Getters and Setters
function SimpleSK:SetAddMethod(value)
    if value then
        self.addMethod = "random"
    else
        self.addMethod = "append"
    end
end

function SimpleSK:GetAddMethod()
    -- Long winded but this might change in future
    local state = false
    if self.addMethod == "random" then
        state = true
    end

    return state
end

function SimpleSK:SetActiveViewMethod(value)
    self.activeViewMethod = value
end

function SimpleSK:GetActiveViewMethod()
    return self.activeViewMethod
end

function SimpleSK:SetSuppressMessageState(value)
    self.suppressMessage = value
end

function SimpleSK:GetSuppressMessageState()
    return self.suppressMessage
end

function SimpleSK:SetListFontSize(value)
    self.listFontSize = value
    self:RefreshWindow()
end

function SimpleSK:GetListFontSize()
    return self.listFontSize
end

function SimpleSK:InputFrame(returnFn, okFn)
    function self.inputTextField.Event:KeyType(button)
        if button and string.byte(button) == 13 then
            returnFn()
        end
    end

    function self.inputOkButton.Event:LeftPress()
        okFn()
    end

    self.inputWindow:SetVisible(true)
    self.baseNameWindow:SetVisible(true)
end

function SimpleSK:ConfigViewListClick()
    if self.configSelectedListName and not (self.configSelectedListName == "") then
        self:SetupSelectNameClick(self.configSelectedListName)
        if self.currentSetupListName and string.len(self.currentSetupListName) > 0 and not (self.currentSetupListName == self.setupSelectFrame:GetSelectedItem()) then
            self.setupSelectFrame:SetSelectedItem(self.currentSetupListName)
        end

        -- Refresh the content
        self:RefreshSetupWindow()

        -- Remember which window was showing last
        self.windowState = "setup"

        -- Hide active frame
        self.activeFrame:SetVisible(false)

        -- Force visibility of setup frame
        self.setupFrame:SetVisible(true)
        self.base:SetVisible(true)

    end
end

function SimpleSK:ConfigAddListClick()
    -- Set focus and selection
    self.inputTextField:SetKeyFocus(true)
    local newName = self:GenerateUniqueListName("NewList", 0)
    self.inputTextField:SetText(newName)
    self.inputTextField:SetSelection(0, string.len(self.inputTextField:GetText()))

    local returnFn = function()
        -- Hide the window
        SimpleSK.inputWindow:SetVisible(false)
        SimpleSK.baseNameWindow:SetVisible(false)

        -- Grab the data and run the function
        SimpleSK:AddList(SimpleSK.inputTextField:GetText())

        -- Clear the field and focus
        SimpleSK.inputTextField:SetText("")
        SimpleSK.inputTextField:SetKeyFocus(false)

        -- Refresh the content
        SimpleSK:RefreshConfigFrame()
    end

    -- Pop up the textfield window
    self:InputFrame(returnFn, returnFn)
end

function SimpleSK:ConfigCopyListClick()
    if not self.configSelectedListName or self.configSelectedListName == "" then return end
    self.copyListName = self.configSelectedListName

    -- Set focus and selection
    self.inputTextField:SetKeyFocus(true)
    local newName = self:GenerateUniqueListName(self.configSelectedListName .. "_", 0)
    self.inputTextField:SetText(newName)
    self.inputTextField:SetSelection(0, string.len(self.inputTextField:GetText()))

    local returnFn = function()
        -- Store the original data
        local origName = SimpleSK.copyListName
        if not origName then return end

        -- Grab the data
        local destName = SimpleSK.inputTextField:GetText()

        -- Run the function
        SimpleSK:CopyList(origName, destName)
        origName = nil

        -- Hide the window
        SimpleSK.inputWindow:SetVisible(false)
        SimpleSK.baseNameWindow:SetVisible(false)

        -- Clear the field and focus
        SimpleSK.inputTextField:SetText("")
        SimpleSK.inputTextField:SetKeyFocus(false)

        -- Clear the old copyList
        SimpleSK.copyListName = nil

        -- Refresh the data
        SimpleSK:RefreshConfigFrame()
    end

    -- Pop up the textfield window
    self:InputFrame(returnFn, returnFn)

    -- Refresh the config frame to clear the selection
    self:RefreshConfigFrame()
end

function SimpleSK:CopyList(origName, destName)
    self:AddList(destName)
    local origList = self:GetListByName(origName, self.pLists)
    local destList = self:GetListByName(destName, self.pLists)
    for i, v in ipairs(origList) do
        table.insert(destList, v)
    end

    self:SetupSelectNameClick(destName)
end

function SimpleSK:GenerateUniqueListName(baseString, counter)
    local listNames = {}
    for k, v in pairs(self.pLists) do
        table.insert(listNames, k)
    end

    local newName = baseString .. tostring(counter)

    local match = true
    while match do
        match = false
        for idx, name in ipairs(listNames) do
            if name == newName then
                match = true
                counter = counter + 1
                newName = baseString .. tostring(counter)
                break
            end
        end
    end

    return newName
end


function SimpleSK:ConfigRenameListClick()
    if not self.configSelectedListName or self.configSelectedListName == "" then return end

    -- Set focus and selection
    self.inputTextField:SetText(self.configSelectedListName)
    self.inputTextField:SetKeyFocus(true)
    self.inputTextField:SetSelection(0, string.len(self.inputTextField:GetText()))

    local returnFn = function()
        -- Store the original data
        local origName = SimpleSK.configSelectedListName
        if not origName then return end

        -- Grab the data
        local destName = SimpleSK.inputTextField:GetText()

        -- Run the function
        SimpleSK:RenameList(origName, destName)

        -- Clear the old list
        origName = nil

        -- Hide the window
        SimpleSK.inputWindow:SetVisible(false)
        SimpleSK.baseNameWindow:SetVisible(false)

        -- Clear the field and focus
        SimpleSK.inputTextField:SetText("")
        SimpleSK.inputTextField:SetKeyFocus(false)

        -- Refresh the window
        SimpleSK:RefreshConfigFrame()
    end

    -- Pop up the textfield window
    self:InputFrame(returnFn, returnFn)

    -- Refresh the config frame to clear the selection
    self:RefreshConfigFrame()
end

function SimpleSK:RenameList(origName, destName)
    if origName == destName then return end

    self:AddList(destName)
    local origList = self:GetListByName(origName, self.pLists)
    local destList = self:GetListByName(destName, self.pLists)
    for i, v in ipairs(origList) do
        table.insert(destList, v)
    end

    self.pLists[origName] = nil
end

function SimpleSK:ConfigImportListClick()
    -- Set focus and selection
    self.inputTextField:SetKeyFocus(true)
    local newName = self:GenerateUniqueListName("NewList", 0)
    self.inputTextField:SetText(newName)
    self.inputTextField:SetSelection(0, string.len(self.inputTextField:GetText()))

    local returnFn = function()
        -- Grab the data
        local destName = SimpleSK.inputTextField:GetText()

        -- Run the function
        SimpleSK:ImportList(destName)
        SimpleSK.configFrame:SetVisible(true)
        SimpleSK.base:SetVisible(true)

        -- Hide the window
        SimpleSK.inputWindow:SetVisible(false)
        SimpleSK.baseNameWindow:SetVisible(false)

        -- Clear the field and focus
        SimpleSK.inputTextField:SetText("")
        SimpleSK.inputTextField:SetKeyFocus(false)
    end

    -- Pop up the textfield window
    self:InputFrame(returnFn, returnFn)
end

function SimpleSK:ConfigDeleteListClick()
    if not self.configSelectedListName or self.configSelectedListName == "" then return end
    local origName = self.configSelectedListName

    self:DeleteList(origName)
    origName = nil

    self:RefreshConfigFrame()
end

function SimpleSK:RefreshAll()
    self:RefreshConfigFrame()
end

function SimpleSK:ShowConfigFrame()
    -- Refresh the content
    self:RefreshConfigFrame()

    -- Force visibility of config frame
    self.configFrame:SetVisible(true)
    self.baseConfigWindow:SetVisible(true)
end

function SimpleSK:ToggleConfigFrame()
    local configState = self.baseConfigWindow:GetVisible()

    if configState then
        self.baseConfigWindow:SetVisible(false)
    else
        self:ShowConfigFrame()
    end

end

function SimpleSK:ScrollToTop(listFrame, listMask)
    self.scrollYOffset = 0
    listFrame:SetPoint("TOPLEFT", listMask, "TOPLEFT", 0, 0)
end

function SimpleSK:ToggleSelectAll(frame)
    if frame:GetChecked() then
        local templist = {}
        for i, _ in ipairs(self.pList) do
            table.insert(templist, i)
        end
        self.currentList = templist

    else
        self.currentList = {}
    end

    self:UpdateLists(self.currentLists, self.currentSetupListName, self.currentList)

    self:RefreshCheckboxesByIndex(self.currentList, self.setupTable)
end

function SimpleSK:RefreshRaidNamesOnEvent(eventUnitTable)
    local match = false

    -- Force GetRaidNames if playerListByID is nil
    if not self.playerListByID then self:GetRaidNames() end

    -- See if the unit changed matches any of my raid units
    for eventUnit, unitValue in pairs(eventUnitTable) do
        for unitInRaid, idValue in pairs(self.playerListByID) do
            if eventUnit == unitInRaid then
                match = true
                break
            end
        end
        if match then
            break
        end
    end

    if match then
        self:GetRaidNames()
        self:GenerateSetupTable(SimpleSK.pList)
    end

end

function SimpleSK:UndoLastEvent()
    -- Abort if there is nothing to undo
    if #self.undoList < 1 then
        return
    end

    -- Grab the last event on the list
    local event = self.undoList[#self.undoList]

    -- Get the lists we are working with
    local oList = self:GetListByName(event.listName, self.oLists)
    local iList = self:GetListByName(event.listName, self.iLists)
    local pList = self:GetListByName(event.listName, self.pLists)

    -- Undo the action on the ordered list
    self:RepositionIndex(oList, event.destination, event.origin)

    -- Update the primary list
    self:UpdatePrimaryList(pList, iList, oList)

    -- Success, pop the last undo event off the list of undo events
    table.remove(self.undoList, #self.undoList)

    -- Update the screen
    self:RefreshWindow()
end

table.insert(Command.Slash.Register("ssk"), {SlashHandler, "SimpleSK", "Slash Handler"})
table.insert(Event.Addon.Load.End, {Initialize, "SimpleSK", "Initialize"})
table.insert(Event.Addon.SavedVariables.Save.Begin, {function() SimpleSK:SaveVars() end, "SimpleSK", "Save variables"})
table.insert(Event.Unit.Availability.Full, {function(eventUnitTable)
        SimpleSK:RefreshRaidNamesOnEvent(eventUnitTable)
    end, "SimpleSK", "Group change"})

table.insert(Event.Unit.Availability.None, {function(eventUnitTable)
        SimpleSK:RefreshRaidNamesOnEvent(eventUnitTable)
    end, "SimpleSK", "Group change"})